using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Halo3Map.Map;
using Halo3Map.Data;
using Halo3Map.Tags;
using Halo3Map.BLF;
using MetaEditor;
using Halo3Map;
using Halo3Map.Scripting;
using Halo3Map.IO;
using System.IO;
using System.Xml;

namespace ForgePC.Forms
{
    public partial class Map_Editor : Form
    {
        public Map map;
        public mapImage mapimage;
        public MetaView metaeditor;        
        public LibraryManager librarymanager;
        private int selectedIndex = -1;

        private bool metaeditoropen;
        private bool viewmetatree;
        private bool viewtoolswindow;

        public Map_Editor(string fileName)
        {
            InitializeComponent();            
            LoadMap(fileName);

            InitialiseLibraryManager();
            LoadLibrarys(".");

            viewmetatree = true;
            viewtoolswindow = true;

            metaeditoropen = false;
            metaEditorButton_Click(null, null); // Enable the Meta Editor.
        }

        private void InitialiseLibraryManager()
        {
            librarymanager = new LibraryManager();

            string path = Application.StartupPath + "\\Libraries";

#if (DEBUG)
            path = string.Format("{0}\\Release\\Libraries", 
                Path.GetDirectoryName(Application.StartupPath));
#endif

            librarymanager.Library.Add(path);
        }

        public void LoadLibrarys(string type)
        {
            for (int i = 0; i < librarymanager.Library.Count; i++)
            {
                if (librarymanager.Library[i].TagType == "." && type == ".")
                {                    
                    //pluginsToolStripMenuItem.DropDownItems.Add(librarymanager.Library[i].GetName());
                }
                else if (librarymanager.Library[i].TagType == type)
                {
                    //pluginsToolStripMenuItem.DropDownItems.Add(librarymanager.Library[i].GetName());
                }
            }

            //Hardcoded Plugins
            if (type == "scnr")
            {
                //scriptEditorToolStripMenuItem.Visible = true;
            }
            else
            {
                //scriptEditorToolStripMenuItem.Visible = false;
            }
        }

        /// <summary>
        /// Loads Map into Map Editer
        /// </summary>
        /// <param name="fileName"></param>
        private void LoadMap(string fileName)
        {
            map = new Map(fileName);
            map.Open();
            map.Read();
            //If load map image setting is enabled
            if (AppSettings.Settings.LoadMapImage == true)
            {
                //Try to load map image
                try
                {
                    mapimage = new mapImage(MiscFunctions.GetMapInfoFileName(map.Header.map_name, map.Header.map_type));
                    mapimage.Read();
                    pictureBox1.Image = mapimage.image;
                }
                catch 
                {
#if (DEBUG)
                    MessageBox.Show("Error Loading Map Preview");
#endif
                }
            }

            //Set text of form labels
            lblName.Text += " " + map.Header.map_name;
            lblPMagic.Text += " " + map.Magic.ToString("X");
            lblMSize.Text += " " + map.Header.map_size.ToString();
            lblType.Text += " " + map.Header.map_type.ToString();
            lblChecksum.Text += " " + map.Header.checksum.ToString("X");
            //If this is a shared map dont go any further
            if (map.Header.map_type == Map_Type.shared ||
                map.Header.map_type == Map_Type.single_player_shared)
                return;
            //Fill the treeview
            FillTreeview();
        }

        /// <summary>
        /// Fills the treview with tags
        /// </summary>
        private void FillTreeview()
        {

            treeView1.BeginUpdate();
            treeView1.SuspendLayout();

            List<TreeNode> tnl = new List<TreeNode>();

            for (int x = 0; x < map.Index.tag_type_count; x++)
            {
                TreeNode tn = new TreeNode(map.Index.MetaClasses[x].type);
                tn.Tag = -1;
                tnl.Add(tn);
            }

            for (int x = 0; x < map.Index.meta_count; x++)
            {
                Meta meta = map.Index.Metas[x];
                if (meta.tagClassIndex == -1)
                    continue;
                TreeNode parent = tnl[meta.tagClassIndex];
                TreeNode tn = new TreeNode(meta.FilePath);
                tn.Tag = x;
                parent.Nodes.Add(tn);
            }

            for (int x = 0; x < tnl.Count; x++)
            {
                if (tnl[x].Nodes.Count > 0)
                    treeView1.Nodes.Add(tnl[x]);
            }

            treeView1.ResumeLayout();
            treeView1.EndUpdate();

            tnl.Clear();
            tnl = null;
        }

        /// <summary>
        /// After a meta is selected in the treeview take the appropriate action
        /// </summary>
        private void treeView1_AfterSelect_1(object sender, TreeViewEventArgs e)
        {
            //Figure out our index
            selectedIndex = (int)e.Node.Tag;
            if (selectedIndex == -1)
                return;

            //Grab our meta out of the list
            Meta meta = map.Index.Metas[selectedIndex];

            //Show Meta Info
            lblClass.Text = "Class: " + meta.TagType;
            lblOffset.Text = "Offset: " + meta.Offset.ToString();
            lblIdent.Text = "Ident: " + meta.ID.ToString("X");

            //Get our tag, hardcoded or plugin
            H3Tag tag = TagHandler.GetTag(meta.TagType, TagHandler.TagType.Plugin);

            //We could not locate a plugin for this tag
            if (tag == null)
                return;

            //Read the meta
            tag.Map = map;
            tag.Read(map.IO.In, meta.Offset);
            LoadLibrarys(tag.TagType);
            //If Meta Editor is open load tag in it
            if (metaeditor != null)
            if (metaeditoropen == true)
            {
                splitContainer1.Panel2.Controls.Clear();
                metaeditor = new MetaView(tag);
                metaeditor.LoadControls();
                metaeditor.Dock = DockStyle.Fill;
                splitContainer1.Panel2.Controls.Add(metaeditor);
            }
        }       
       
        /// <summary>
        /// Fixes the maps checksum
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnResign_Click(object sender, EventArgs e)
        {
            //Resgin Map
            map.Resign();
            //Display new Checksum
            lblChecksum.Text = "Checksum:" + map.Header.checksum.ToString();
        }
       
        /// <summary>
        /// Extracts meta to a file
        /// </summary>
        private void extractBubbleButton_Click(object sender, DevComponents.DotNetBar.ClickEventArgs e)
        {
            if (selectedIndex == -1) return; // No tag selected.

            //Grab our meta out of the list
            Meta meta = map.Index.Metas[selectedIndex];

            //Get our tag, hardcoded or plugin
            H3Tag tag = TagHandler.GetTag(meta.TagType, TagHandler.TagType.HardCoded);

            //We could not locate a plugin for this tag
            if (tag == null)
            {
                MessageBox.Show("Error: There is no plugin for this tag type");
                return;
            }
            //Set up Save File Dialog
            SaveFileDialog sfd = new SaveFileDialog();
            //Set filepath of meta
            sfd.FileName = meta.FilePath;
            //Set Filter to meta type
            sfd.Filter = tag.TagType.ToUpper() +  " Meta File (*." + tag.TagType + ")|*." + tag.TagType;
            //Show Save File Dialog
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                tag.Map = map;
                tag.Read(map.IO.In, meta.Offset);

                FileStream fs = new FileStream(sfd.FileName, FileMode.Create);
                EndianWriter bw = new EndianWriter(fs, EndianType.BigEndian);
                XmlTextWriter xtw = new XmlTextWriter(sfd.FileName.Replace("." + tag.TagType, ".xml"), Encoding.Default);
                
                tag.Extract(bw, xtw);

                xtw.Close();
                bw.Close();
                fs.Close();

                MessageBox.Show("Done");
            }
        }
       
        /// <summary>
        /// Inject meta from a file
        /// </summary>
        private void injectBubbleButton_Click(object sender, DevComponents.DotNetBar.ClickEventArgs e)
        {
            if (selectedIndex == -1) return; // No tag selected.

            //Grab our meta out of the list
            Meta meta = map.Index.Metas[selectedIndex];

            //Get our tag, hardcoded or plugin
            H3Tag tag = TagHandler.GetTag(meta.TagType, TagHandler.TagType.HardCoded);

            //We could not locate a plugin for this tag
            if (tag == null)
            {
                MessageBox.Show("Error: There is no plugin for this tag type");
                return;
            }

            //Set up Open File Dialog
            OpenFileDialog ofd = new OpenFileDialog();
            //Set filepath of meta
            ofd.FileName = meta.FilePath;
            //Set Filter to meta type
            ofd.Filter = tag.TagType.ToUpper() + " Meta File (*." + tag.TagType + ")|*." + tag.TagType;
            //Show Open File Dialog
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                tag.Map = map;
                tag.Read(map.IO.In, meta.Offset);

                FileStream fs = new FileStream(ofd.FileName, FileMode.Open);
                EndianReader br = new EndianReader(fs, EndianType.BigEndian);

                tag.Inject(br);

                br.Close();
                fs.Close();

                MessageBox.Show("Done");
            }
        } 
     
        /// <summary>
        /// Copys the value of a label to the clipboard
        /// </summary>
        /// <param name="label">Label to copy value from</param>
        /// <param name="dec">Decimal or Hexadecimal</param>
        public void CopyToValueClipBoard(Label label, bool dec)
        {
            //Split string 
            string[] temp = label.Text.Split(' ');
            
            //If converting to decimal
            if (dec == true)
            {
                #region If Decimal
                try
                {
                    Clipboard.SetText(int.Parse(temp[temp.Length - 1]).ToString());
                }
                #endregion
                #region If HexaDecimal
                catch
                {
                    Clipboard.SetText(uint.Parse(temp[temp.Length - 1], System.Globalization.NumberStyles.AllowHexSpecifier).ToString());
                }
                #endregion
            }
            //If converting to hexadecimal
            else
            {
                #region If Decimal
                try
                {
                    Clipboard.SetText(int.Parse(temp[temp.Length - 1]).ToString("X"));
                }
                #endregion
                #region If HexaDecimal
                catch
                {
                    Clipboard.SetText(temp[temp.Length - 1]);
                }
                #endregion
            }
            
        }

        private void copyValueToClipboardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CopyToValueClipBoard((Label)contextMenuStrip1.SourceControl, true);            
        }

        private void copyHexiToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CopyToValueClipBoard((Label)contextMenuStrip1.SourceControl, false);
        }

        private void toolsButton_Click(object sender, DevComponents.DotNetBar.ClickEventArgs e)
        {
            if (viewtoolswindow == true)
            {
                splitContainer1.Panel2Collapsed = true;
                toolsButton.Image = ForgePC.Properties.Resources.tools;
                toolsButton.ImageLarge = ForgePC.Properties.Resources.tools;
                viewtoolswindow = false;
            }
            else
            {
                splitContainer1.Panel2Collapsed = false;
                toolsButton.Image = ForgePC.Properties.Resources.toolsselected;
                toolsButton.ImageLarge = ForgePC.Properties.Resources.toolsselected;
                viewtoolswindow = true;
            }
            bubbleBar1.Refresh();
        } 

        private void metaTreeButton_Click(object sender, DevComponents.DotNetBar.ClickEventArgs e)
        {
            if (viewmetatree == true)
            {
                splitContainer1.Panel1Collapsed = true;
                metaTreeButton.Image = ForgePC.Properties.Resources.metatree;
                metaTreeButton.ImageLarge = ForgePC.Properties.Resources.metatree;
                viewmetatree = false;
            }
            else
            {
                splitContainer1.Panel1Collapsed = false;
                metaTreeButton.Image = ForgePC.Properties.Resources.metatreeselected;
                metaTreeButton.ImageLarge = ForgePC.Properties.Resources.metatreeselected;
                viewmetatree = true;
            }
            bubbleBar1.Refresh();
        } 

        private void metaEditorButton_Click(object sender, DevComponents.DotNetBar.ClickEventArgs e)
        {
            if (metaeditoropen == false)
            {
                splitContainer1.Panel2.Controls.Clear();
                metaeditor = new MetaView();
                metaeditor.Dock = DockStyle.Fill;
                splitContainer1.Panel2.Controls.Add(metaeditor);
                metaEditorButton.Image = ForgePC.Properties.Resources.metaeditorSelected;
                metaEditorButton.ImageLarge = ForgePC.Properties.Resources.metaeditorSelected;
                metaeditoropen = true;                
            }
            else
            {
                splitContainer1.Panel2.Controls.Clear();
                metaeditor.Dispose();
                metaEditorButton.Image = ForgePC.Properties.Resources.metaeditor;
                metaEditorButton.ImageLarge = ForgePC.Properties.Resources.metaeditor;
                metaeditoropen = false;
            }

            bubbleBar1.Refresh();
        }                     
    }
}