using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using ForgePC.IFP;
using ForgePC.Map;

namespace entity.Forms.Meta_Editor
{
    public partial class Enums : UserControl
    {
        #region Fields
        public int LineNumber;
        private int mapIndex;
        public int chunkOffset;
        public int offsetInMap;
        public int enumType;
        public int Value;
        public string EntName = "Error in getting plugin element name";
        private bool isNulledOutReflexive = true;
        private object[] options;
        #endregion
        #region Constructor
        public Enums(string iEntName, int map, int iOffsetInChunk, int iType, object[] ioptions, int iLineNumber)
        {
            this.LineNumber = iLineNumber;
            this.options = ioptions;
            this.enumType = iType;
            this.chunkOffset = iOffsetInChunk;
            mapIndex = map;
            this.EntName = iEntName;
            InitializeComponent();
            this.Size = this.PreferredSize;
            this.Dock = DockStyle.Top;
            this.Controls[1].Text = EntName;
            switch (iType)
            {
                case 8:
                    {
                        this.Controls[2].Text = "Enum8";
                        break;
                    }
                case 16:
                    {
                        this.Controls[2].Text = "Enum16";
                        break;
                    }
                case 32:
                    {
                        this.Controls[2].Text = "Enum32";
                        break;
                    }
            }
            this.AutoSize = false;
        }
        #endregion
        #region Load and Save
        private void UpdateComboBox()
        {
            //add all the values from the options list to a temp list<string>.
            List<int> tempint = new List<int>(0);
            if (this.options != null)
            {
                for (int counter = 0; counter < this.options.Length; counter++)
                {
                    tempint.Add(((IFPIO.Option)this.options[counter]).value);
                }
            }
            //see if the value in the map is in the list, if not, add it.
            if (tempint.IndexOf(this.Value) == -1)
                tempint.Add(this.Value);
            tempint.TrimExcess();
            tempint.Sort();
            int tempindex = tempint.IndexOf(this.Value);
            //go through and put the option name in the strings after the ": "
            List<string> tempstring = new List<string>(0);
            for (int counter = 0; counter < tempint.Count; counter++)
            {
                tempstring.Add(tempint[counter].ToString());
            }
            if (this.options != null)
            {
                for (int counter = 0; counter < this.options.Length; counter++)
                {
                    if (tempint.IndexOf(((IFPIO.Option)this.options[counter]).value) != -1)
                        tempstring[tempint.IndexOf(((IFPIO.Option)this.options[counter]).value)] += ": " + ((IFPIO.Option)this.options[counter]).name.ToString();
                }
            }
            ((ComboBox)this.Controls[0]).Items.Clear();
            ((ComboBox)this.Controls[0]).Items.AddRange(tempstring.ToArray());
            ((ComboBox)this.Controls[0]).SelectedIndex = tempindex;
        }        
        public void Populate(int iOffset)
        {
            this.isNulledOutReflexive = false;
            bool openedMap = false;
            if (Maps.map[mapIndex].isOpen == false)
            {
                Maps.map[mapIndex].Open(H3Dev.IO.EndianType.BigEndian, H3Dev.Map.H3Map.MapType.Internal);
                openedMap = true;
            }
            Maps.map[mapIndex].io.In.BaseStream.Position = iOffset + this.chunkOffset;
            switch (this.enumType)
            {
                case 8:
                    {
                        this.Value = (int)Maps.map[mapIndex].io.In.ReadByte();
                        break;
                    }
                case 16:
                    {
                        this.Value = (int)Maps.map[mapIndex].io.In.ReadInt16();
                        break;
                    }
                case 32:
                    {
                        this.Value = Maps.map[mapIndex].io.In.ReadInt32();
                        break;
                    }

            }
            this.offsetInMap = iOffset + this.chunkOffset;
            UpdateComboBox();
            if (openedMap == true)
                Maps.map[mapIndex].Close();
        }
        public void Save()
        {
            if (this.isNulledOutReflexive == true)
                return;
            bool openedMap = false;
            if (Maps.map[mapIndex].isOpen == false)
            {
                Maps.map[mapIndex].Open(H3Dev.IO.EndianType.BigEndian, H3Dev.Map.H3Map.MapType.Internal);
                openedMap = true;
            }
            this.UpdateValue();
            Maps.map[mapIndex].io.Out.BaseStream.Position = this.offsetInMap;
            try
            {
                switch (this.enumType)
                {
                    case 8:
                        {
                            Maps.map[mapIndex].io.Out.Write(Convert.ToByte(this.Value));
                            break;
                        }
                    case 16:
                        {
                            Maps.map[mapIndex].io.Out.Write(Convert.ToInt16(this.Value));
                            break;
                        }
                    case 32:
                        {
                            Maps.map[mapIndex].io.Out.Write(this.Value);
                            break;
                        }
                }
            }
            catch
            {
                MessageBox.Show("Something is wrong with this enum " + this.EntName + " Offset " + this.chunkOffset.ToString());
            }
            if (openedMap == true)
                Maps.map[mapIndex].Close();
        }
        //public void Poke()
        //{
        //    uint Address = (uint)(this.offsetInMap + Maps.map[mapIndex].SelectedMeta.Magic);
        //    this.UpdateValue();
        //    switch (this.enumType)
        //    {
        //        case 8:
        //            {
        //                Feline_rawr.RTH_Imports.Poke(Address, (uint)Convert.ToByte(this.Value), 8);
        //                break;
        //            }
        //        case 16:
        //            {
        //                Feline_rawr.RTH_Imports.Poke(Address, (uint)Convert.ToInt16(this.Value), 16);
        //                break;
        //            }
        //        case 32:
        //            {
        //                Feline_rawr.RTH_Imports.Poke(Address, (uint)this.Value, 32);
        //                break;
        //            }
        //    }
        //}
        private void UpdateValue()
        {
            //Since I mixed the name with the value, we have to take out the names.
            string tempstring = ((ComboBox)this.Controls[0]).Text;
            for (int counter = 0; counter < tempstring.Length; counter++)
            {
                if (tempstring[counter] == ':')
                {
                    tempstring = tempstring.Substring(0, counter);
                    break;
                }
            }
            //if some n00b (read xbox7887) decided to type something stupid into the enum box, here we just test to see if it'll work.
            //if not, the value stays the same as what was read in
            try
            {
                this.Value = Convert.ToInt32(tempstring);
            }
            catch
            {
            }
        }
        #endregion
        public void SetFocus(int LineToCheck)
        {
            if (this.LineNumber == LineToCheck)
                this.Focus();
        }
    }
}
