using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using ForgePC.IFP;
using ForgePC.Map;

namespace entity.Forms.Meta_Editor
{
    public partial class Indices : UserControl
    {        
        #region Fields
        public int LineNumber;
        private IFPIO.Index EntIndex;
        private List<string> tempvalue;
        private int mapIndex;
        public int chunkOffset;
        public int offsetInMap;
        private string selectionBoxValueString;
        private int indexedReflexiveChunkCount;
        private int indexedReflexiveTranslatedOffset;
        private DataValues.ENTType ItemType;
        public DataValues.ENTType ValueType;
        public int Value;
        public string EntName = "Error in getting plugin element name";
        private int indexedReflexiveOffset;
        private bool isNulledOutReflexive = true;
        #endregion
        public Indices(string iEntName, int map, int iOffsetInChunk, IFPIO.ObjectEnum iValueType, IFPIO.Index iIndex)
        {
            this.LineNumber = iIndex.lineNumber;
            this.Dock = DockStyle.Top;            
            this.mapIndex = map;
            this.chunkOffset = iOffsetInChunk;
            this.EntName = iEntName;
            this.EntIndex = iIndex;
            this.tempvalue = new List<string>(0);
            #region Set Block Indice Reference Type
            switch (this.EntIndex.ItemType.ToLower())
            {
                case "ident":
                    {
                        this.ItemType = entity.Forms.Meta_Editor.DataValues.ENTType.ident;
                        break;
                    }
                case "stringid":
                    {
                        this.ItemType = entity.Forms.Meta_Editor.DataValues.ENTType.sid;
                        break;
                    }
                case "float":
                    {
                        this.ItemType = entity.Forms.Meta_Editor.DataValues.ENTType.Float;
                        break;
                    }
                case "short":
                    {
                        this.ItemType = entity.Forms.Meta_Editor.DataValues.ENTType.Int16;
                        break;
                    }
                case "ushort":
                    {
                        this.ItemType = entity.Forms.Meta_Editor.DataValues.ENTType.Uint16;
                        break;
                    }
                case "int":
                    {
                        this.ItemType = entity.Forms.Meta_Editor.DataValues.ENTType.Int32;
                        break;
                    }
                case "uint":
                    {
                        this.ItemType = entity.Forms.Meta_Editor.DataValues.ENTType.UInt32;
                        break;
                    }
                case "string32":
                    {
                        this.ItemType = entity.Forms.Meta_Editor.DataValues.ENTType.string32;
                        break;
                    }
                case "unicodestring64":
                    {
                        this.ItemType = entity.Forms.Meta_Editor.DataValues.ENTType.unicodestring64;
                        break;
                    }
                case "string256":
                    {
                        this.ItemType = entity.Forms.Meta_Editor.DataValues.ENTType.string256;
                        break;
                    }
                case "unicodestring256":
                    {
                        goto case "string256";
                    }
                default:
                    {
                        this.ItemType = entity.Forms.Meta_Editor.DataValues.ENTType.nothing;
                        break;
                    }
            }
            #endregion
            InitializeComponent();
            switch (iValueType)
            {
                case IFPIO.ObjectEnum.Short:
                    {
                        this.Controls[1].Text = "Int16 Block Index";
                        this.ValueType = DataValues.ENTType.Int16;
                        break;
                    }
                case IFPIO.ObjectEnum.Int:
                    {
                        this.Controls[1].Text = "Int32 Block Index";
                        this.ValueType = DataValues.ENTType.Int32;
                        break;
                    }
                case IFPIO.ObjectEnum.UShort:
                    {
                        this.Controls[1].Text = "Uint16 Block Index";
                        this.ValueType = DataValues.ENTType.Uint16;
                        break;
                    }
                case IFPIO.ObjectEnum.UInt:
                    {
                        this.Controls[1].Text = "Uint32 Block Index";
                        this.ValueType = DataValues.ENTType.UInt32;
                        break;
                    }
                case IFPIO.ObjectEnum.Byte:
                    {
                        this.Controls[1].Text = "Byte Block Index";
                        this.ValueType = DataValues.ENTType.b;
                        break;
                    }
            }
            this.Controls[0].Text = this.EntName;
        }
        public void Populate(int iOffset,int iIndexedReflexiveOffset)
        {
            this.isNulledOutReflexive = false;
            if (this.EntIndex.reflexiveLayer.ToLower() == "root")
                this.indexedReflexiveOffset = Maps.map[mapIndex].SelectedMeta.Offset + this.EntIndex.ReflexiveOffset;
            else if (this.EntIndex.reflexiveLayer.ToLower() == "oneup")
                this.indexedReflexiveOffset = iIndexedReflexiveOffset + this.EntIndex.ReflexiveOffset;
            bool openedMap = false;
            if (Maps.map[mapIndex].isOpen == false)
            {
                Maps.map[mapIndex].Open(H3Dev.IO.EndianType.BigEndian, H3Dev.Map.H3Map.MapType.Internal);
                openedMap = true;
            }
            Maps.map[mapIndex].io.In.BaseStream.Position = iOffset + this.chunkOffset;
            this.offsetInMap = iOffset + this.chunkOffset;
            switch (ValueType)
            {
                case DataValues.ENTType.Int16:
                    {
                        this.Value = (int)Maps.map[mapIndex].io.In.ReadInt16();
                        break;
                    }
                case DataValues.ENTType.Int32:
                    {
                        this.Value = Maps.map[mapIndex].io.In.ReadInt32();
                        break;
                    }
                case DataValues.ENTType.Uint16:
                    {
                        this.Value = (int)Maps.map[mapIndex].io.In.ReadUInt16();
                        break;
                    }
                case DataValues.ENTType.UInt32:
                    {
                        this.Value = (int)Maps.map[mapIndex].io.In.ReadUInt32();
                        break;
                    }
                case DataValues.ENTType.b:
                    {
                        this.Value = (int)Maps.map[mapIndex].io.In.ReadByte();
                        break;
                    }
            }
            UpdateSelectionList(false);
            if (openedMap == true)
                Maps.map[mapIndex].Close();
        }
        private void UpdateSelectionList(bool readAll)
        {
            this.ReadReflexive();
            this.GetItems(readAll);
            ((ComboBox)this.Controls[2]).Items.Clear();
            ((ComboBox)this.Controls[2]).Items.AddRange(tempvalue.ToArray());

            if (this.Value >= this.indexedReflexiveChunkCount)
            {
                ((ComboBox)this.Controls[2]).Items.Add(this.Value.ToString() + " : Value is Too Large To Be The Indexer");
                ((ComboBox)this.Controls[2]).SelectedIndex = ((ComboBox)this.Controls[2]).Items.Count - 1;
            }
            else if (this.Value < -1)
            {
                ((ComboBox)this.Controls[2]).Items.Add(this.Value.ToString() + " : Value is Too Small To Be An Index");
                ((ComboBox)this.Controls[2]).SelectedIndex = ((ComboBox)this.Controls[2]).Items.Count - 1;
            }
            else if (this.Value == -1)
            {
                if (readAll == false)
                    ((ComboBox)this.Controls[2]).Text = "nulled";
                else
                    ((ComboBox)this.Controls[2]).SelectedIndex = this.indexedReflexiveChunkCount;
            }
            else
            {
                if (readAll == false)
                    ((ComboBox)this.Controls[2]).Text = this.selectionBoxValueString;
                else
                    ((ComboBox)this.Controls[2]).SelectedIndex = this.Value;
            }
        }
        private void ReadReflexive()
        {
            bool openedMap = false;
            if (Maps.map[mapIndex].isOpen == false)
            {
                Maps.map[mapIndex].Open(H3Dev.IO.EndianType.BigEndian, H3Dev.Map.H3Map.MapType.Internal);
                openedMap = true;
            }
            Maps.map[mapIndex].io.In.BaseStream.Position = this.indexedReflexiveOffset;
            this.indexedReflexiveChunkCount = Maps.map[mapIndex].io.In.ReadInt32();
            this.indexedReflexiveTranslatedOffset = Maps.map[mapIndex].io.In.ReadInt32() - Maps.map[mapIndex].SelectedMeta.Magic;
            if (openedMap == true)
                Maps.map[mapIndex].Close();
        }
        private void GetItems(bool readAll)
        {
            this.tempvalue.Clear();
            if (this.indexedReflexiveChunkCount < 0)
            {
                tempvalue.Add("Index " + this.EntName + " Is Invalid. On Line " + this.EntIndex.lineNumber.ToString());
                return;
            }
            bool openedMap = false;
            if (Maps.map[mapIndex].isOpen == false)
            {
                Maps.map[mapIndex].Open(H3Dev.IO.EndianType.BigEndian, H3Dev.Map.H3Map.MapType.Internal);
                openedMap = true;
            }            
            int countervalue = 0;
            int maxcount = this.indexedReflexiveChunkCount;
            if (readAll == false && this.Value > -1)
            {
                countervalue = this.Value;
                maxcount = this.Value + 1;
            }
            else if (readAll == false)
            {
                countervalue = 0;
                maxcount = 0;
            }
            string tempValueString = "";
            for (int counter = countervalue; counter < maxcount; counter++)
            {
                try
                {
                    Maps.map[mapIndex].io.In.BaseStream.Position = this.indexedReflexiveTranslatedOffset + (this.EntIndex.ReflexiveSize * counter) + this.EntIndex.ItemOffset;
                    switch (this.EntIndex.ItemType.ToLower())
                    {
                        case "short":
                            {
                                tempValueString = counter.ToString() + " : " + Maps.map[mapIndex].io.In.ReadInt16().ToString();
                                break;
                            }
                        case "int":
                            {
                                tempValueString = counter.ToString() + " : " + Maps.map[mapIndex].io.In.ReadInt32().ToString();
                                break;
                            }
                        case "ushort":
                            {
                                tempValueString = counter.ToString() + " : " + Maps.map[mapIndex].io.In.ReadUInt16().ToString();
                                break;
                            }
                        case "uint":
                            {
                                tempValueString = counter.ToString() + " : " + Maps.map[mapIndex].io.In.ReadUInt32().ToString();
                                break;
                            }
                        case "float":
                            {
                                tempValueString = counter.ToString() + " : " + Maps.map[mapIndex].io.In.ReadSingle().ToString();
                                break;
                            }
                        //case "stringid":
                        //    {
                        //        int sidIndexer = Maps.map[mapIndex].BR.ReadInt16();
                        //        if (sidIndexer > -1 && sidIndexer < Maps.map[mapIndex].Strings.Name.Length)
                        //        {
                        //            tempValueString = counter.ToString() + " : " + Maps.map[mapIndex].Strings.Name[sidIndexer];
                        //        }
                        //        else
                        //        {
                        //            tempValueString = counter.ToString() + " : Not a String ID";
                        //        }
                        //        break;
                        //    }
                        //case "ident":
                        //    {                                
                        //        int tempint = Maps.map[mapIndex].BR.ReadInt32();                                
                        //        int tagIndex = Maps.map[mapIndex].Functions.Meta.FindMetaByID(tempint, mapIndex);
                        //        if (tagIndex != -1)
                        //        {
                        //            if (tempint != -1)
                        //            {
                        //                tempValueString = counter.ToString() + " : " + Maps.map[mapIndex].MetaInfo.TagType[tagIndex] + " - " + Maps.map[mapIndex].FileNames.Name[tagIndex];
                        //            }
                        //            else
                        //            {
                        //                tempValueString = counter.ToString() + " : Probably is NOT an ident, choose another data type";
                        //            }
                        //        }
                        //        else if(tagIndex == -1)
                        //        {
                        //            tempValueString = counter.ToString() + " : nulled ident";
                        //        }
                        //        else
                        //        {
                        //            tempValueString = counter.ToString() + " : Probably is NOT an ident, choose another data type";
                        //        }
                        //        break;
                        //    }
                        case "string32":
                            {
                                Encoding decode = Encoding.UTF8;
                                byte[] tempbytes = Maps.map[mapIndex].io.In.ReadBytes(32);
                                tempValueString = counter.ToString() + " : " + decode.GetString(tempbytes);
                                break;
                            }
                         case "unicodestring64":
                            {
                                Encoding decode = Encoding.UTF8;
                                byte[] tempbytes = Maps.map[mapIndex].io.In.ReadBytes(64);
                                tempValueString = counter.ToString() + " : " + decode.GetString(tempbytes);
                                break;
                            }
                        case "string256":
                            {
                                Encoding decode = Encoding.UTF8;
                                byte[] tempbytes = Maps.map[mapIndex].io.In.ReadBytes(256);
                                tempValueString = counter.ToString() + " : " + decode.GetString(tempbytes);
                                break;
                            }
                        case "unicodestring256":
                            {
                                goto case "string256";
                            }

                    }
                }
                catch
                {
                    tempValueString = counter.ToString() + " : Error with index on line number " + this.EntIndex.lineNumber.ToString();
                }
                tempvalue.Add(tempValueString);
                if (counter == this.Value)
                    this.selectionBoxValueString = tempValueString;
            }
            tempvalue.Add("nulled");
            if (openedMap == true)
                Maps.map[mapIndex].Close();
        }
        public void Save()
        {
            if (this.isNulledOutReflexive == true)
                return;
            string tempstring1 = ((ComboBox)this.Controls[2]).Text;
            if (tempstring1.Contains(" Is Invalid. On Line ") || tempstring1.Contains("Something is wrong with this ") || tempstring1.Contains(" : Value is Too Small To Be An Index") || tempstring1.Contains(" : Value is Too Large To Be The Indexer"))
                return;
            if (tempstring1 == "nulled")
                this.Value = -1;
            if (tempstring1.Contains(" : "))
            {
                int counter;
                for (counter = 0; counter < tempstring1.Length; counter++)
                {
                    if (tempstring1[counter] == ' ')
                    {
                        break;
                    }
                }
                this.Value = Convert.ToInt32(tempstring1.Substring(0, counter));
            }          
            bool openedMap = false;
            if (Maps.map[mapIndex].isOpen == false)
            {
                Maps.map[mapIndex].Open(H3Dev.IO.EndianType.BigEndian, H3Dev.Map.H3Map.MapType.Internal);
                openedMap = true;
            }
            try
            {
                Maps.map[mapIndex].io.Out.BaseStream.Position = this.offsetInMap;
                switch (ValueType)
                {
                    case DataValues.ENTType.Int16:
                        {
                            Maps.map[mapIndex].io.Out.Write(Convert.ToInt16(this.Value));
                            break;
                        }
                    case DataValues.ENTType.Int32:
                        {
                            Maps.map[mapIndex].io.Out.Write(Convert.ToInt32(this.Value));
                            break;
                        }
                    case DataValues.ENTType.Uint16:
                        {
                            Maps.map[mapIndex].io.Out.Write(Convert.ToUInt16(this.Value));
                            break;
                        }
                    case DataValues.ENTType.UInt32:
                        {
                            Maps.map[mapIndex].io.Out.Write(Convert.ToUInt32(this.Value));
                            break;
                        }
                    case DataValues.ENTType.Float:
                        {
                            Maps.map[mapIndex].io.Out.Write(Convert.ToSingle(this.Value));
                            break;
                        }
                    case DataValues.ENTType.undefined:
                        {
                            Maps.map[mapIndex].io.Out.Write(Convert.ToSingle(this.Value));
                            break;
                        }
                    case DataValues.ENTType.b:
                        {
                            Maps.map[mapIndex].io.Out.Write(Convert.ToByte(this.Value));
                            break;
                        }
                }
            }
            catch
            {
                MessageBox.Show("Something is wrong with this " + this.ValueType.ToString() + this.EntName + " Offset " + this.chunkOffset.ToString());
            }
            if (openedMap == true)
                Maps.map[mapIndex].Close();
        }
        //public void Poke()
        //{
        //    if (this.isNulledOutReflexive == true)
        //        return;
        //    string tempstring1 = ((ComboBox)this.Controls[2]).Text;
        //    if (tempstring1.Contains(" Is Invalid. On Line ") || tempstring1.Contains("Something is wrong with this ") || tempstring1.Contains(" : Value is Too Small To Be An Index") || tempstring1.Contains(" : Value is Too Large To Be The Indexer"))
        //        return;
        //    if (tempstring1 == "nulled")
        //        this.Value = -1;
        //    if (tempstring1.Contains(" : "))
        //    {
        //        int counter;
        //        for (counter = 0; counter < tempstring1.Length; counter++)
        //        {
        //            if (tempstring1[counter] == ' ')
        //            {
        //                break;
        //            }
        //        }
        //        this.Value = Convert.ToInt32(tempstring1.Substring(0, counter));
        //    }
        //    uint Address = (uint)(this.offsetInMap + Maps.map[mapIndex].SelectedMeta.Magic);
        //    switch (ValueType)
        //    {
        //        case DataValues.ENTType.Int16:
        //            {
        //                Feline_rawr.RTH_Imports.Poke(Address, (uint)Convert.ToInt16(this.Value), 16);
        //                break;
        //            }
        //        case DataValues.ENTType.Int32:
        //            {
        //                Feline_rawr.RTH_Imports.Poke(Address, (uint)Convert.ToInt32(this.Value), 32);
        //                break;
        //            }
        //        case DataValues.ENTType.Uint16:
        //            {
        //                Feline_rawr.RTH_Imports.Poke(Address, (uint)Convert.ToUInt16(this.Value), 16);
        //                break;
        //            }
        //        case DataValues.ENTType.UInt32:
        //            {
        //                Feline_rawr.RTH_Imports.Poke(Address, (uint)Convert.ToUInt32(this.Value), 32);
        //                break;
        //            }
        //        case DataValues.ENTType.Float:
        //            {
        //                Feline_rawr.RTH_Imports.Poke(Address, (uint)Feline_rawr.RTH_Imports.ConvertFloat(Convert.ToSingle(this.Value)), 32);
        //                break;
        //            }
        //        case DataValues.ENTType.undefined:
        //            {
        //                Feline_rawr.RTH_Imports.Poke(Address, (uint)Feline_rawr.RTH_Imports.ConvertFloat(Convert.ToSingle(this.Value)), 32);
        //                break;
        //            }
        //        case DataValues.ENTType.b:
        //            {
        //                Feline_rawr.RTH_Imports.Poke(Address, (uint)Convert.ToByte(this.Value), 8);
        //                break;
        //            }
        //    }
        //}

        private void comboBox1_DropDown(object sender, EventArgs e)
        {
            UpdateSelectionList(true);
        }
        public void SetFocus(int LineToCheck)
        {
            if (this.LineNumber == LineToCheck)
                this.Focus();
        }
    }
}
