using System;
using System.Collections.Generic;
using System.Text;

using Xceed.Zip;
using System.IO;
using System.Windows.Forms;
using System.Net;
using System.Xml;

namespace ForgePC
{
    public class Updating
    {
        static string UpdateXmlLocation = "http://h3engineer.exofire.net/Engineer/update.xml";
        static string localUpdateName = "\\update.zip";

        public event DownloadProgressChangedEventHandler DownloadProgressChangedEventHandler;
        public event System.ComponentModel.AsyncCompletedEventHandler AsyncCompletedEventHandler;

        public static bool isUpdateAvaiable(double CurrentVersion)
        {
            //Get out update information
            UpdateInfo ui = GetUpdateInfo();

            //Return if its a newer version
            return (ui.UpdateVersion > CurrentVersion);
        }

        public void DownloadUpdate()
        {
            //Get out update information
            UpdateInfo ui = GetUpdateInfo();

            //Create a webclient
            WebClient wc = new WebClient();
            wc.DownloadProgressChanged += DownloadProgressChangedEventHandler;
            wc.DownloadFileCompleted += AsyncCompletedEventHandler;

            //Begin our download
            wc.DownloadFileAsync(new Uri(ui.ZipURL), Application.StartupPath + localUpdateName);
        }

        public static UpdateInfo GetUpdateInfo()
        {
            //Create the update info struct
            UpdateInfo ui = new UpdateInfo();

            //Download our data and read it
            XmlTextReader xtr = new XmlTextReader(UpdateXmlLocation);

            //Read the version
            xtr.ReadToFollowing("version");
            ui.UpdateVersion = xtr.ReadElementContentAsDouble();

            //Read the update url
            xtr.ReadToFollowing("updateURL");
            ui.ZipURL = xtr.ReadElementContentAsString();

            //Return our update info
            return ui;
        }

        public void UnpackUpdate()
        {

            //Rename all the files to be updated
            QuickZipItem[] items = QuickZip.GetZipContents(Application.StartupPath + localUpdateName, "*");
            for (int i = 0; i < items.Length; i++)
            {
                string name = items[i].FullName;
                if (File.Exists(Application.StartupPath + name))
                    File.Move(Application.StartupPath + name,
                        Application.StartupPath + name + ".old");
            }

            //Unzip the file
            QuickZip.Unzip(Application.StartupPath + localUpdateName, Application.StartupPath + "\\", true, true, true, "*");

        }

        public void DeleteUpdate()
        {
            File.Delete(Application.StartupPath + localUpdateName);
        }

        public static void PerformCleanup()
        {
            DirectoryInfo di = new DirectoryInfo(Application.StartupPath);
            PerformCleanup(di);
        }

        public static void PerformCleanup(DirectoryInfo di)
        {
            DirectoryInfo[] ds = di.GetDirectories();
            for (int i = 0; i < ds.Length; i++)
            {
                PerformCleanup(ds[i]);
            }

            FileInfo[] fs = di.GetFiles();
            for (int i = 0; i < fs.Length; i++)
            {
                string ext = fs[i].Extension;
                if (ext == ".old")
                {
                    fs[i].Delete();
                }
            }
        }

        public struct UpdateInfo
        {
            public string ZipURL;
            public double UpdateVersion;
        }

    }
}