using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Collections;
using ForgePC;
using System.Xml;
using Halo3Map.IO;

namespace H3Dev.Map
{
    public class H3Map
    {
        #region Fields
        public string file_name;
        public bool isOpen;
        public Meta SelectedMeta;
        public EndianIO io;
        public Map_Header map_header;
        public String_ID string_id;
        public File_Name file_names;
        public Index_Header index_header;
        public Index_Items index_items;
        #endregion
        #region Constructor
        public H3Map(string file_loc)
        {
            file_name = file_loc;
            Open(EndianType.BigEndian, MapType.Internal);

            map_header = new Map_Header(ref io, this);
            string_id = new String_ID(ref io, this);
            file_names = new File_Name(ref io, this);
            index_header = new Index_Header(ref io, this);
            index_items = new Index_Items(ref io, this);           
        }
        #endregion
        #region Methods
        public enum MapType : int
        {
            MainMenu,
            Shared = 0,
            Internal,
        }

        public void Open(EndianType IOType, MapType type)
        {
            switch (IOType)
            {
                case EndianType.BigEndian:
                    switch (type)
                    {
                        case MapType.Internal:
                            io = new EndianIO(file_name, EndianType.BigEndian);
                            io.Open();
                            break;
                        case MapType.Shared:
                            OpenFileDialog ofd = new OpenFileDialog();
                            if (ofd.ShowDialog() == DialogResult.OK)
                                io = new EndianIO(ofd.FileName, EndianType.BigEndian);
                            io.Open();
                            break;
                    }
                    break;
                case EndianType.LittleEndian:
                    switch (type)
                    {
                        case MapType.Internal:
                            io = new EndianIO(file_name, EndianType.LittleEndian);
                            io.Open();
                            break;
                    }
                    break;
            }
            isOpen = true;
        }

        public void Close()
        {
 
        }

        public void Sign()
        {
            int result = 0;
            int x;
            io.In.BaseStream.Position = 12288;
            int temp = ((int)io.In.BaseStream.Length - 12288) / 4;
            for (x = 0; x < temp; x++)
            {
                result = result ^ io.In.ReadInt32();
            }
            io.Out.BaseStream.Seek(708, SeekOrigin.Begin);
            io.Out.Write(result);
            map_header.checksum = result;
        }

        public int findIndexNumber(string Class, string Name)
        {
            for (int i = 0; i < index_header.meta_count; i++)
            {
                if (Class == index_items.Class[i] && Name == index_items.Name[i])
                {
                    return i;
                }
            }
            return -1;
        }

        public int findIndexNumberByID(int Idt)
        {
            for (int i = 0; i < index_header.meta_count; i++)
            {
                if (Idt == index_items.Ident[i])
                {
                    return i;
                }
            }
            return -1;
        }

        public int FindMetaByOffset(int offset)
        {
            for (int i = 0; i < index_header.meta_count; i++)
            {
                if ((offset >= index_items.Offset[i]) && (offset < (index_items.Offset[i] + index_items.Size[i])))
                {
                    return i;
                }
            }
            return -1;
        }

        public void PopulateTreeView(TreeView tv)
        {
            ArrayList temp = new ArrayList();
            for (int i = 0; i < index_header.meta_count; i++)
            {
                if (!temp.Contains(index_items.Class[i]))
                {
                    temp.Add(index_items.Class[i]);
                    TreeNode tempn = new TreeNode(index_items.Class[i]);
                    for (int x = 0; x < index_header.meta_count; x++)
                    {
                        if (index_items.Class[x] == index_items.Class[i])
                        {
                            TreeNode tempn2 = new TreeNode(index_items.Name[x]);
                            tempn.Nodes.Add(tempn2);
                        }
                    }
                    tv.Nodes.Add(tempn);
                }
            }

            tv.Sort();
        }
        #endregion
        #region Classes
        #region Header
        public class Map_Header
        {
            public string head; //0
            public version build; //4
            public int map_size; //8
            public int memory_modifier; //16
            public int index_header_offset; //16
            public string time_stamp; //284
            public Map_Type map_type; //314
            public int string_id_count; //344
            public int string_id_table_offset; //not defined in header
            public string map_name; //396
            public string scenario_name; //432
            public int file_table_count; //692
            public int file_table_offset; //not defined in header
            public int file_index_offset; //not defined in header
            public int checksum; //708
            public string foot; //12284

            public int address_mask_constant;
            public int address_mask;

            public Map_Header(ref EndianIO IO, H3Map map)
            {
                IO.In.BaseStream.Position = 0;
                head = new string(IO.In.ReadChars(4));
                build = (version)IO.In.ReadInt32();
                map_size = IO.In.ReadInt32();
                IO.In.BaseStream.Position = 16;
                memory_modifier = IO.In.ReadInt32();
                IO.In.BaseStream.Position = 284;
                time_stamp = new string(IO.In.ReadChars(32));
                IO.In.BaseStream.Position = 314;
                map_type = (Map_Type)IO.In.ReadInt32();
                IO.In.BaseStream.Position = 344;
                string_id_count = IO.In.ReadInt32();
                IO.In.BaseStream.Position = 396;
                char temp1;
                while ((temp1 = IO.In.ReadChar()) != '\0')
                {
                    map_name += temp1;
                }
                IO.In.BaseStream.Position = 432;
                char temp2;
                while ((temp2 = IO.In.ReadChar()) != '\0')
                {
                    scenario_name += temp2;
                }
                IO.In.BaseStream.Position = 692;
                file_table_count = IO.In.ReadInt32();
                IO.In.BaseStream.Position = 708;
                checksum = IO.In.ReadInt32();
                IO.In.BaseStream.Position = 12284;
                foot = new string(IO.In.ReadChars(4));

                address_mask_constant = -1076731904;
                address_mask = address_mask_constant - map_size;
                index_header_offset = memory_modifier - address_mask;
            }

            public enum version : int
            {
                halo_3_beta = 9,
                halo_3_retail = 11
            }

            public enum Map_Type : int
            {
                single_player = 0,
                multi_player = 1,
                mainmenu = 2,
                shared = 3,
                single_player_shared = 4
            }
        }
        #endregion
        #region String Id
        public class String_ID
        {
            public int[] table_offset;
            public string[] name;

            public String_ID(ref EndianIO IO, H3Map map)
            {
                IO.In.BaseStream.Position = 12288;
                table_offset = new int[map.map_header.string_id_count];
                for (int i = 0; i < map.map_header.string_id_count; i++)
                {
                    table_offset[i] = IO.In.ReadInt32();
                }

                map.map_header.string_id_table_offset = (int)IO.In.BaseStream.Position + MiscFunctions.Padding(map.map_header.string_id_count * 4, 2048);

                name = new string[map.map_header.string_id_count];
                for (int i = 0; i < map.map_header.string_id_count; i++)
                {
                    IO.In.BaseStream.Position = map.map_header.string_id_table_offset + table_offset[i];
                    char temp1;
                    while ((temp1 = IO.In.ReadChar()) != '\0')
                    {
                        name[i] += temp1;
                    }
                }

                int table_size = (int)IO.In.BaseStream.Position - map.map_header.string_id_table_offset;
                map.map_header.file_table_offset = (int)IO.In.BaseStream.Position + MiscFunctions.Padding(table_size, 4096);
                IO.In.BaseStream.Position = map.map_header.file_table_offset;
                if (IO.In.ReadInt32() == 0)
                    map.map_header.file_table_offset += 2048;
            }
        }
        #endregion
        #region File Names
        public class File_Name
        {
            public int[] table_offset;
            public string[] name;

            public File_Name(ref EndianIO IO, H3Map map)
            {
                name = new string[map.map_header.file_table_count];

                IO.In.BaseStream.Position = map.map_header.file_table_offset;
                for (int i = 0; i < map.map_header.file_table_count; i++)
                {
                    char temp1;
                    while ((temp1 = IO.In.ReadChar()) != '\0')
                    {
                        name[i] += temp1;
                    }                    
                }

                int table_size = (int)IO.In.BaseStream.Position - map.map_header.file_table_offset;
                map.map_header.file_index_offset = (int)IO.In.BaseStream.Position + MiscFunctions.Padding(table_size, 4096);

                /*IO.In.BaseStream.Position = map.map_header.file_index_offset;
                if (IO.In.ReadInt16 != 100)
                    map.map_header.file_table_offset += 2048*/


                table_offset = new int[map.map_header.file_table_count];
                IO.In.BaseStream.Position = map.map_header.file_index_offset;
                for (int i = 0; i < map.map_header.file_table_count; i++)
                {
                    table_offset[i] = IO.In.ReadInt32();
                }
            }
        }
        #endregion
        #region Index Header
        public class Index_Header
        {


            public int primary_magic;

            public Index_Header(ref EndianIO IO, H3Map map)
            {
                primary_magic = map.map_header.memory_modifier - map.map_header.index_header_offset;

                IO.In.BaseStream.Position = map.map_header.index_header_offset;
                tag_type_count = IO.In.ReadInt32();
                tag_type_index_offset = IO.In.ReadInt32() - primary_magic;
                meta_count = IO.In.ReadInt32();
                tag_info_offset = IO.In.ReadInt32() - primary_magic;
                unkown_six = IO.In.ReadInt32();
                main_tags = IO.In.ReadInt32() - primary_magic;
                unkown_zero = IO.In.ReadInt32();
                main_tags_2 = IO.In.ReadInt32() - primary_magic;
                unknown = IO.In.ReadInt32();
            }            
        }
        #endregion
        #region Index Items
        public class Index_Items
        {
            public Tag_Class_Index tag_classes;

            public string[] Class;
            public string[] Name;
            public int[] Ident;
            public int[] Offset;
            public int[] Size;

            public Index_Items(ref EndianIO IO, H3Map map)
            {
                tag_classes = new Tag_Class_Index(ref IO, map);

                Class = new string[map.index_header.meta_count];
                Name = new string[map.index_header.meta_count];
                Ident = new int[map.index_header.meta_count];
                Offset = new int[map.index_header.meta_count];
                Size = new int[map.index_header.meta_count];

                IO.In.BaseStream.Position = map.index_header.tag_info_offset;
                for (int i = 0; i < map.index_header.meta_count; i++)
                {
                    Class[i] = tag_classes.class_type[IO.In.ReadInt16()];
                    Name[i] = map.file_names.name[i];
                    Ident[i] = IO.In.ReadInt16();
                    Offset[i] = IO.In.ReadInt32() - map.index_header.primary_magic;
                    //Size[i] = Meta_Sizes.Get_Size(Class[i]);
                } 
            }

            public class Tag_Class_Index
            {
                public string[] class_type;

                public Tag_Class_Index(ref EndianIO IO, H3Map map)
                {
                    class_type = new string[map.index_header.tag_type_count];

                    for (int i = 0; i < class_type.Length; i++)
                    {
                        IO.In.BaseStream.Position = map.index_header.tag_type_index_offset + (16 * i);
                        class_type[i] = new string(IO.In.ReadChars(4));
                    }
                }
            }
        }
        #endregion
        #endregion
    }
}
