using System;
using System.Collections.Generic;
using System.Text;
using ForgePC.Map;
using ForgePC.IFP;

namespace Halo3Map.Meta_Editor
{
    public static class Base_Object
    {
        #region Base
        public class Base
        {
            public Object_Type type;
            public bool visible;
            public string name;
            public string description;
            public int offset;
            public int mapnumber;

            public Base()
            {

            }
        }
        #endregion
        #region Enums
        public enum Object_Type
        {
            Int,
            Short,
            Byte,
            Float,
            Bitmask,
            Enum,
            Reflex,
            Tagname,
            Ident,
            String,
            StringID,
            Unused,
        }
        #endregion
        #region Base Types
        public class Int : Base
        {
            #region Fields
            public int value;
            #endregion
            #region Constructor
            public Int(string Name, int map_offset, int map_number, bool isvisible)
            {
                type = Object_Type.Int;
                name = Name;
                offset = map_offset;
                mapnumber = map_number;
                visible = isvisible;
            }
            #endregion
            #region Methods
            public void Load()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                value = Maps.map[mapnumber].io.In.ReadInt32();
            }

            public void Save()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                Maps.map[mapnumber].io.Out.Write(value);
            }
            #endregion
        }

        public class Short : Base
        {
            #region Fields
            public short value;
            #endregion
            #region Constructor
            public Short(string Name, int map_offset, int map_number, bool isvisible)
            {
                type = Object_Type.Short;
                name = Name;
                offset = map_offset;
                mapnumber = map_number;
                visible = isvisible;
            }
            #endregion
            #region Methods
            public void Load()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                value = Maps.map[mapnumber].io.In.ReadInt16();
            }

            public void Save()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                Maps.map[mapnumber].io.Out.Write(value);
            }
            #endregion
        }
        public class Byte : Base
        {
            #region Fields
            public byte value;
            #endregion
            #region Constructor
            public Byte(string Name, int map_offset, int map_number, bool isvisible)
            {
                type = Object_Type.Byte;
                name = Name;
                offset = map_offset;
                mapnumber = map_number;
            }
            #endregion
            #region Methods
            public void Load()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                value = Maps.map[mapnumber].io.In.ReadByte();
            }

            public void Save()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                Maps.map[mapnumber].io.Out.Write(value);
            }
            #endregion
        }
        public class Float : Base
        {
            #region Fields
            public float value;
            #endregion
            #region Constructor
            public Float(string Name, int map_offset, int map_number, bool isvisible)
            {
                type = Object_Type.Float;
                name = Name;
                offset = map_offset;
                mapnumber = map_number;
            }
            #endregion
            #region Methods
            public void Load()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                value = Maps.map[mapnumber].io.In.ReadSingle();
            }

            public void Save()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                Maps.map[mapnumber].io.Out.Write(value);
            }
            #endregion
        }

        public class Enum : Base
        {
            #region Fields
            public string[] options;
            public int value;
            #endregion
            #region Constructor
            public Enum(string Name, int map_offset, int map_number, object[] option_names, bool isvisible)
            {
                type = Object_Type.Enum;
                name = Name;
                offset = map_offset;
                mapnumber = map_number;
                for (int i = 0; i < option_names.Length; i++)
                {
                    options[i] = (string)option_names[i];
                }
            }
            #endregion
            #region Methods
            public void Load()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                value = Maps.map[mapnumber].io.In.ReadInt16();
            }

            public void Save()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                Maps.map[mapnumber].io.Out.Write(value);
            }
            #endregion
        }

        public class Bitmask : Base
        {
            #region Fields
            public int bitcount;
            public object value;
            #endregion
            #region Constructor
            public Bitmask(string Name, int map_offset, int bit_count, int map_number, bool isvisible)
            {
                type = Object_Type.Bitmask;
                name = Name;
                offset = map_offset;
                mapnumber = map_number;

                bitcount = bit_count;
            }
            #endregion
            #region Methods
            public void Load()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                switch (this.bitcount)
                {
                    case 8:
                        {
                            value = Maps.map[mapnumber].io.In.ReadByte();
                            break;
                        }
                    case 16:
                        {
                            value = Maps.map[mapnumber].io.In.ReadUInt16();
                            break;
                        }
                    case 32:
                        {
                            value = Maps.map[mapnumber].io.In.ReadUInt32();
                            break;
                        } 
                }
            }
            #endregion
        }

        public class Tag_Name : Base
        {
            #region Fields
            public string tag_class;
            #endregion
            #region Constructor
            public Tag_Name(string Name, int map_offset, int map_number, bool isvisible)
            {
                type = Object_Type.Tagname;
                name = Name;
                offset = map_offset;
                mapnumber = map_number;
            }
            #endregion
            #region Methods
            public void Load()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                tag_class = new string(Maps.map[mapnumber].io.In.ReadChars(4));
            }

            public void Save()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                char[] temp = tag_class.ToCharArray();
                Maps.map[mapnumber].io.Out.Write(temp);
            }
            #endregion
        }

        public class Ident : Base
        {
            #region Fields
            public bool loneID;
            public short ident;
            public string tagtype;
            public string filename;
            #endregion
            #region Constructor
            public Ident(string Name, int map_offset, bool isLoneID, int map_number, bool isvisible)
            {
                type = Object_Type.Ident;
                name = Name;
                offset = map_offset;
                loneID = isLoneID;
                mapnumber = map_number;
            }
            #endregion
            #region Methods
            public void Load()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                ident = Maps.map[mapnumber].io.In.ReadInt16();
                if (ident != -1)
                {
                    tagtype = Maps.map[mapnumber].index_items.Class[Maps.map[mapnumber].findIndexNumberByID(ident)];
                    filename = Maps.map[mapnumber].index_items.Name[Maps.map[mapnumber].findIndexNumberByID(ident)];
                }
            }

            public void Save()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                Maps.map[mapnumber].io.Out.Write(ident);
            }
            #endregion
        }
        public class String : Base
        {
            #region Fields
            public string value;
            #endregion
            #region Constructor
            public String(string Name, int map_offset, int map_number, bool isvisible)
            {
                type = Object_Type.String;
                name = Name;
                offset = map_offset;
                mapnumber = map_number;
            }
            #endregion
            #region Methods
            public void Load()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                value = new string(Maps.map[mapnumber].io.In.ReadChars(32));
            }

            public void Save()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                Maps.map[mapnumber].io.Out.Write(name.ToCharArray());
            }
            #endregion
        }
        public class Reflex : Base
        {
            #region Fields
            public int chunksize;
            public int chunkcount;
            public int translation;
            public object[] elements;
            public List<Base_Object.Base> items;
            #endregion
            #region Constructor
            public Reflex(string Name, int map_offset, int chunk_size, int map_number, object[] Elements, bool isvisible)
            {
                type = Object_Type.Reflex;
                name = Name;
                offset = map_offset;
                chunksize = chunk_size;
                mapnumber = map_number;
                elements = Elements;
            }
            #endregion
            #region Methods
            public void Load()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                chunkcount = Maps.map[mapnumber].io.In.ReadInt32();
                translation = Maps.map[mapnumber].io.In.ReadInt32() - Maps.map[mapnumber].index_header.primary_magic;
            }
            /*public void Save()
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = offset;
                Maps.map[mapnumber].io.Out.Write(value);
            }*/
            #endregion
        }
        #endregion
    }
}
