using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using H3Dev.Map;

namespace Halo3Map.Meta_Editor.Controls
{
    public partial class Meta_Editor : UserControl
    {
        private Meta m;

        public Meta_Editor(Meta me)
        {
            InitializeComponent();
            m = me;
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            Save();
            MessageBox.Show("Success");
        }

        #region Load Controls
        public void Load_Controls()
        {
            foreach(Base_Object.Base b in m.Items)
            {
                switch (b.type)
                {
                    case Base_Object.Object_Type.Int:
                        Base_Object.Int tempint = (Base_Object.Int)b;
                        Int i = new Int(tempint);
                        i.Dock = DockStyle.Top;
                        this.Controls.Add(i);
                        this.Controls[Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.Short:
                        Base_Object.Short tempshort = (Base_Object.Short)b;
                        Short sh = new Short(tempshort);
                        sh.Dock = DockStyle.Top;
                        this.Controls.Add(sh);
                        this.Controls[Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.Float:
                        Base_Object.Float tempfloat = (Base_Object.Float)b;
                        Float f = new Float(tempfloat);
                        f.Dock = DockStyle.Top;
                        this.Controls.Add(f);
                        this.Controls[Controls.Count - 1].BringToFront();                     
                        break;
                    case Base_Object.Object_Type.Byte:
                        Base_Object.Byte tempbyte = (Base_Object.Byte)b;
                        Byte bb = new Byte(tempbyte);
                        bb.Dock = DockStyle.Top;
                        this.Controls.Add(bb);
                        this.Controls[Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.Enum:
                        Base_Object.Enum tempenum = (Base_Object.Enum)b;
                        Enum e = new Enum(tempenum);
                        e.Dock = DockStyle.Top;
                        this.Controls.Add(e);
                        this.Controls[Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.String:
                        Base_Object.String tempstring = (Base_Object.String)b;
                        String s = new String(tempstring);
                        s.Dock = DockStyle.Top;
                        this.Controls.Add(s);
                        this.Controls[Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.Ident:
                        Base_Object.Ident tempident = (Base_Object.Ident)b;
                        Ident id = new Ident(tempident);
                        id.Dock = DockStyle.Top;                      
                        this.Controls.Add(id);
                        this.Controls[Controls.Count - 1].BringToFront();                        
                        break;
                    case Base_Object.Object_Type.Reflex:
                        Base_Object.Reflex tempreflex = (Base_Object.Reflex)b;
                        if (tempreflex.chunkcount != 0)
                        {
                            Reflexive r = new Reflexive(tempreflex);
                            r.Dock = DockStyle.Top;
                            this.Controls.Add(r);
                            this.Controls[Controls.Count - 1].BringToFront();                            
                        }
                        break;                    
                }
            }
        }
        #endregion
        #region Save
        public void Save()
        {
            foreach (Control c in this.Controls)
            {
                switch (c.GetType().ToString())
                {
                    case "Halo3Map.Meta_Editor.Controls.Byte":
                        ((Byte)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.Int":
                        ((Int)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.Short":
                        ((Short)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.Enum":
                        ((Enum)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.Ident":
                        ((Ident)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.Reflexive":
                        ((Reflexive)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.String":
                        ((String)c).Save();
                        break;
                    default:
                        if(c.GetType().ToString() != "System.Windows.Forms.Panel")
                            MessageBox.Show("Error " + c.GetType().ToString() + " was not saved");
                        break;
                }
            }
        }
        #endregion
    }
}
