using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using ForgePC.IFP;

namespace Halo3Map.Meta_Editor.Controls
{
    public partial class Reflexive : UserControl
    {
        public bool visible;
        public List<Halo3Map.Meta_Editor.Base_Object.Base> Items;
        public Base_Object.Reflex re;        

        public Reflexive(Base_Object.Reflex reflex)
        {
            InitializeComponent();
            re = reflex;
            label1.Text = reflex.name;
            for (int i = 0; i < reflex.chunkcount; i++)
            {
                comboBox1.Items.Add(i);
            }
            textBox1.Text = reflex.chunkcount.ToString();
            textBox2.Text = reflex.translation.ToString();

            visible = reflex.visible;
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            Clear_Controls();
            Load_Chunk(comboBox1.SelectedIndex);
            Load_Controls();
        }

        public void Clear_Controls()
        {
            panel2.Controls.Clear();
        }

        #region Load Chunk
        public void Load_Chunk(int chunk)
        {
            Items = new List<Base_Object.Base>();

            foreach (object o in re.elements)
            {
                IFPIO.BaseObject tempbase = (IFPIO.BaseObject)o;

                switch (tempbase.ObjectType)
                {
                    case IFPIO.ObjectEnum.Int:
                        IFPIO.IFPInt tInt = (IFPIO.IFPInt)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Int Bint = new Halo3Map.Meta_Editor.Base_Object.Int(tInt.name, re.translation + (chunk * re.chunksize) + tInt.offset, re.mapnumber, tInt.visible);
                        Bint.Load();
                        Items.Add(Bint);
                        break;
                    case IFPIO.ObjectEnum.Short:
                        IFPIO.IFPInt tempshort = (IFPIO.IFPInt)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Short Bshort = new Halo3Map.Meta_Editor.Base_Object.Short(tempshort.name, re.translation + (chunk * re.chunksize) + tempshort.offset, re.mapnumber, tempshort.visible);
                        Bshort.Load();
                        Items.Add(Bshort);
                        break;
                    case IFPIO.ObjectEnum.Float:
                        IFPIO.IFPFloat tempfloat = (IFPIO.IFPFloat)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Float Bfloat = new Halo3Map.Meta_Editor.Base_Object.Float(tempfloat.name, re.translation + (chunk * re.chunksize) + tempfloat.offset, re.mapnumber, tempfloat.visible);
                        Bfloat.Load();
                        Items.Add(Bfloat);
                        break;
                    case IFPIO.ObjectEnum.Byte:
                        IFPIO.IFPByte tempbyte = (IFPIO.IFPByte)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Byte Bbyte = new Halo3Map.Meta_Editor.Base_Object.Byte(tempbyte.name, re.translation + (chunk * re.chunksize) + tempbyte.offset, re.mapnumber, tempbyte.visible);
                        Bbyte.Load();
                        Items.Add(Bbyte);
                        break;
                    case IFPIO.ObjectEnum.Enum:
                        IFPIO.IFPEnum tempenum = (IFPIO.IFPEnum)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Enum Benum = new Halo3Map.Meta_Editor.Base_Object.Enum(tempenum.name, re.translation + (chunk * re.chunksize) + tempenum.offset, re.mapnumber, tempenum.options, tempenum.visible);
                        Benum.Load();
                        Items.Add(Benum);
                        break;
                    case IFPIO.ObjectEnum.Bitmask:
                        IFPIO.Bitmask tempbitm = (IFPIO.Bitmask)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Bitmask bitmask = new Halo3Map.Meta_Editor.Base_Object.Bitmask(tempbitm.name, re.translation + (chunk * re.chunksize) + tempbitm.offset, tempbitm.bitmaskSize, re.mapnumber, tempbitm.visible);
                        bitmask.Load();
                        Items.Add(bitmask);
                        break;
                    case IFPIO.ObjectEnum.TagType:
                        IFPIO.TagType temptagtype = (IFPIO.TagType)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Tag_Name tagname = new Halo3Map.Meta_Editor.Base_Object.Tag_Name(temptagtype.name, re.translation + (chunk * re.chunksize) + temptagtype.offset, re.mapnumber, temptagtype.visible);
                        tagname.Load();
                        Items.Add(tagname);
                        break;
                    case IFPIO.ObjectEnum.Ident:
                        IFPIO.Ident tempindent = (IFPIO.Ident)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Ident ident = new Halo3Map.Meta_Editor.Base_Object.Ident(tempindent.name, re.translation + (chunk * re.chunksize) + tempindent.offset, tempindent.hasTagType, re.mapnumber, tempindent.visible);
                        ident.Load();
                        Items.Add(ident);
                        break;
                    case IFPIO.ObjectEnum.String32:
                        IFPIO.IFPString tempstring = (IFPIO.IFPString)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.String Bstring = new Halo3Map.Meta_Editor.Base_Object.String(tempstring.name, re.translation + (chunk * re.chunksize) + tempstring.offset, re.mapnumber, tempstring.visible);
                        Bstring.Load();
                        Items.Add(Bstring);
                        break;
                    case IFPIO.ObjectEnum.Reflexive:
                        IFPIO.Reflexive tempreflex = (IFPIO.Reflexive)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Reflex reflex = new Halo3Map.Meta_Editor.Base_Object.Reflex(tempreflex.name, re.translation + (chunk * re.chunksize) + tempreflex.offset, tempreflex.chunkSize, re.mapnumber, tempreflex.items, tempreflex.visible);
                        reflex.Load();
                        Items.Add(reflex);
                        break;
                }
            }
        }
        #endregion
        #region Load Controls
        public void Load_Controls()
        {
            foreach (Base_Object.Base b in this.Items)
            {
                switch (b.type)
                {
                    case Base_Object.Object_Type.Int:
                        Base_Object.Int tempint = (Base_Object.Int)b;
                        Int i = new Int(tempint);
                        i.Dock = DockStyle.Top;
                        panel2.Controls.Add(i);
                        panel2.Controls[panel2.Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.Short:
                        Base_Object.Short tempshort = (Base_Object.Short)b;
                        Short sh = new Short(tempshort);
                        sh.Dock = DockStyle.Top;
                        panel2.Controls.Add(sh);
                        panel2.Controls[panel2.Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.Float:
                        Base_Object.Float tempfloat = (Base_Object.Float)b;
                        Float f = new Float(tempfloat);
                        f.Dock = DockStyle.Top;
                        panel2.Controls.Add(f);
                        panel2.Controls[panel2.Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.Byte:
                        Base_Object.Byte tempbyte = (Base_Object.Byte)b;
                        Byte bb = new Byte(tempbyte);
                        bb.Dock = DockStyle.Top;
                        panel2.Controls.Add(bb);
                        panel2.Controls[panel2.Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.Enum:
                        Base_Object.Enum tempenum = (Base_Object.Enum)b;
                        Enum e = new Enum(tempenum);
                        e.Dock = DockStyle.Top;
                        panel2.Controls.Add(e);
                        panel2.Controls[panel2.Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.String:
                        Base_Object.String tempstring = (Base_Object.String)b;
                        String s = new String(tempstring);
                        s.Dock = DockStyle.Top;
                        panel2.Controls.Add(s);
                        panel2.Controls[panel2.Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.Ident:
                        Base_Object.Ident tempident = (Base_Object.Ident)b;
                        Ident id = new Ident(tempident);
                        id.Dock = DockStyle.Top;
                        panel2.Controls.Add(id);
                        panel2.Controls[panel2.Controls.Count - 1].BringToFront();
                        break;
                    case Base_Object.Object_Type.Reflex:
                        Base_Object.Reflex tempreflex = (Base_Object.Reflex)b;
                        if (tempreflex.chunkcount != 0)
                        {
                            Reflexive r = new Reflexive(tempreflex);
                            r.Dock = DockStyle.Top;
                            panel2.Controls.Add(r);
                            panel2.Controls[panel2.Controls.Count - 1].BringToFront();
                        }
                        break;
                }
            }
        }
        #endregion
        #region Save
        public void Save()
        {
            foreach (Control c in panel2.Controls)
            {
                switch (c.GetType().ToString())
                {
                    case "Halo3Map.Meta_Editor.Controls.Byte":
                        ((Byte)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.Int":
                        ((Int)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.Short":
                        ((Short)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.Enum":
                        ((Enum)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.Ident":
                        ((Ident)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.Reflexive":
                        ((Reflexive)c).Save();
                        break;
                    case "Halo3Map.Meta_Editor.Controls.String":
                        ((String)c).Save();
                        break;
                    default:
                        MessageBox.Show("Error " + c.GetType().ToString() + " was not saved");
                        break;
                }
            }
        }
        #endregion
    }
}
