using System;
using System.Collections.Generic;
using System.Text;
using H3Dev.Map;
using ForgePC.Map;
using System.IO;
using ForgePC.IFP;
using Halo3Map.Map;
using System.Xml;

namespace H3Dev.Map
{
    public class Meta
    {
        #region Fields
        public MemoryStream MS;
        public List<Halo3Map.Meta_Editor.Base_Object.Base> Items;

        public int TagNumber;
        public string Class;
        public string Name;
        public int Offset;
        public int Size;
        public int Ident;
        public int Magic;

        int mapnumber;
        #endregion
        #region Constructor
        public Meta(int tagNumber, int mapNumber)
        {
            TagNumber = tagNumber;
            mapnumber = mapNumber;

            if (TagNumber != -1)
            {
                Class = Maps.map[mapnumber].index_items.Class[TagNumber];
                Name = Maps.map[mapnumber].index_items.Name[TagNumber];
                Offset = Maps.map[mapnumber].index_items.Offset[TagNumber];
                try
                {
                    Size = Calculate_Size();
                }
                catch
                {
                    Size = 0;
                }
                Ident = Maps.map[mapnumber].index_items.Ident[TagNumber];
                Magic = Maps.map[mapnumber].index_header.primary_magic;
            }
        }
        #endregion
        #region Methods
        public int Calculate_Size()
        {
            ForgePC.IFP.IFPHashMap.RemoveIfp(Class, mapnumber);
            IFPIO io = ForgePC.IFP.IFPHashMap.GetIfp(Class, mapnumber);
            
            return Meta_Sizes.Calculate_Size_wIFP(ref io, this, mapnumber);
        }
        public void ReadMetaFromMap()
        {
            MS=new MemoryStream(Size);
            Maps.map[mapnumber].io.In.BaseStream.Position = Offset;
            MS.Write(Maps.map[mapnumber].io.In.ReadBytes(Size), 0, Size);
        }

        #region Meta Scanner
        public void Scan_Meta()
        {
            ForgePC.IFP.IFPHashMap.RemoveIfp(Class, mapnumber);
            IFPIO ifp = ForgePC.IFP.IFPHashMap.GetIfp(Class, mapnumber);

            Items = new List<Halo3Map.Meta_Editor.Base_Object.Base>();
            try
            {
                Scan_Meta_With_IFP(ifp.items, Offset);
            }
            catch { }
        }

        public void Scan_Meta_With_IFP(object[] ifp, int offset)
        {
            foreach (object o in ifp)
            {
                IFPIO.BaseObject tempbase = (IFPIO.BaseObject)o;

                switch (tempbase.ObjectType)
                {
                    case IFPIO.ObjectEnum.Int:
                        IFPIO.IFPInt tempint = (IFPIO.IFPInt)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Int Bint = new Halo3Map.Meta_Editor.Base_Object.Int(tempint.name, offset + tempint.offset, mapnumber, tempint.visible);
                        Bint.Load();
                        Items.Add(Bint);
                        break;
                    case IFPIO.ObjectEnum.Short:
                        IFPIO.IFPInt tempshort = (IFPIO.IFPInt)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Short Bshort = new Halo3Map.Meta_Editor.Base_Object.Short(tempshort.name, offset + tempshort.offset, mapnumber, tempshort.visible);
                        Bshort.Load();
                        Items.Add(Bshort);
                        break;
                    case IFPIO.ObjectEnum.Float:
                        IFPIO.IFPFloat tempfloat = (IFPIO.IFPFloat)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Float Bfloat = new Halo3Map.Meta_Editor.Base_Object.Float(tempfloat.name, offset + tempfloat.offset, mapnumber, tempfloat.visible);
                        Bfloat.Load();
                        Items.Add(Bfloat);
                        break;
                    case IFPIO.ObjectEnum.Byte:
                        IFPIO.IFPByte tempbyte = (IFPIO.IFPByte)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Byte Bbyte = new Halo3Map.Meta_Editor.Base_Object.Byte(tempbyte.name, offset + tempbyte.offset, mapnumber, tempbyte.visible);
                        Bbyte.Load();
                        Items.Add(Bbyte);
                        break;
                    case IFPIO.ObjectEnum.Enum:
                        IFPIO.IFPEnum tempenum = (IFPIO.IFPEnum)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Enum Benum = new Halo3Map.Meta_Editor.Base_Object.Enum(tempenum.name, offset + tempenum.offset, mapnumber, tempenum.options, tempenum.visible);
                        Benum.Load();
                        Items.Add(Benum);
                        break;
                    case IFPIO.ObjectEnum.Bitmask:
                        IFPIO.Bitmask tempbitm = (IFPIO.Bitmask)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Bitmask bitmask = new Halo3Map.Meta_Editor.Base_Object.Bitmask(tempbitm.name, offset + tempbitm.offset, tempbitm.bitmaskSize, mapnumber, tempbitm.visible);
                        bitmask.Load();
                        Items.Add(bitmask);
                        break;
                    case IFPIO.ObjectEnum.TagType:
                        IFPIO.TagType temptagtype = (IFPIO.TagType)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Tag_Name tagname = new Halo3Map.Meta_Editor.Base_Object.Tag_Name(temptagtype.name, offset + temptagtype.offset, mapnumber, temptagtype.visible);
                        tagname.Load();
                        Items.Add(tagname);
                        break;
                    case IFPIO.ObjectEnum.Ident:
                        IFPIO.Ident tempindent = (IFPIO.Ident)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Ident ident = new Halo3Map.Meta_Editor.Base_Object.Ident(tempindent.name, offset + tempindent.offset, tempindent.hasTagType, mapnumber, tempindent.visible);
                        ident.Load();
                        Items.Add(ident);
                        break;
                    case IFPIO.ObjectEnum.String32:
                        IFPIO.IFPString tempstring = (IFPIO.IFPString)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.String Bstring = new Halo3Map.Meta_Editor.Base_Object.String(tempstring.name, offset + tempstring.offset, mapnumber, tempstring.visible);
                        Bstring.Load();
                        Items.Add(Bstring);
                        break;
                    case IFPIO.ObjectEnum.Reflexive:
                        IFPIO.Reflexive tempreflex = (IFPIO.Reflexive)tempbase;
                        Halo3Map.Meta_Editor.Base_Object.Reflex reflex = new Halo3Map.Meta_Editor.Base_Object.Reflex(tempreflex.name, offset + tempreflex.offset, tempreflex.chunkSize, mapnumber, tempreflex.items, tempreflex.visible);
                        reflex.Load();
                        Items.Add(reflex);
                        break;
                }
            }
        }
        #endregion
        #region Extract/Inject
        /// <summary>
        /// Extracts meta to a specifed location
        /// </summary>
        /// <param name="outputfilename">Location to save to</param>
        public void Extract_Meta(string outputfilename)
        {
            outputfilename = outputfilename.Replace("<", "_");
            outputfilename = outputfilename.Replace(">", "_");

            FileStream fs = new FileStream(outputfilename, FileMode.Create);
            BinaryWriter bw = new BinaryWriter(fs);

            ForgePC.IFP.IFPHashMap.RemoveIfp(Class, mapnumber);
            IFPIO ifp = ForgePC.IFP.IFPHashMap.GetIfp(Class, mapnumber);

            XmlTextWriter xtw = new XmlTextWriter(outputfilename + ".xml", Encoding.Default);
            xtw.Formatting = Formatting.Indented;
            xtw.WriteStartElement("Meta");
            xtw.WriteAttributeString("TagType", Class);
            xtw.WriteAttributeString("TagName", Name);
            xtw.WriteAttributeString("Offset", Offset.ToString());
            xtw.WriteAttributeString("Size", Size.ToString());
            xtw.WriteAttributeString("Magic", Magic.ToString());
            xtw.WriteAttributeString("Date", DateTime.Today.ToShortDateString());
            xtw.WriteAttributeString("Time", DateTime.Now.ToShortTimeString());
            xtw.WriteAttributeString("Forge ", "Beta 0.1");

            Maps.map[mapnumber].io.In.BaseStream.Position = Offset;
            byte[] temp = Maps.map[mapnumber].io.In.ReadBytes(ifp.headerSize);

            bw.Write(temp);

            xtw.WriteStartElement("Header");
            xtw.WriteAttributeString("Offset", "0");
            xtw.WriteAttributeString("Offset", ifp.headerSize.ToString());
            xtw.WriteEndElement();


            Extract_Meta_wIFP(ref bw, ref xtw, ifp.items, Offset);

            xtw.WriteEndElement();
            xtw.Close();

            fs.Close();
            bw.Close();
        }
        
        public void Inject_Meta(string imputfilename)
        {
            FileStream fs = new FileStream(imputfilename, FileMode.Open);
            BinaryReader br = new BinaryReader(fs);

            ForgePC.IFP.IFPHashMap.RemoveIfp(Class, mapnumber);
            IFPIO ifp = ForgePC.IFP.IFPHashMap.GetIfp(Class, mapnumber);

            XmlTextReader xtr = new XmlTextReader(imputfilename + ".xml");

            while (xtr.Read())
            {
                switch (xtr.NodeType.ToString())
                {
                    case "Element":
                        {
                            switch (xtr.Name.ToLower())
                            {
                                case "meta":
                                    {
                                        int offset = int.Parse(xtr.GetAttribute("Offset"));
                                        int Size = int.Parse(xtr.GetAttribute("Size"));
                                        br.BaseStream.Position = 0;
                                        byte[] temp = br.ReadBytes(Size);

                                        Maps.map[mapnumber].io.Out.BaseStream.Position = offset;
                                        Maps.map[mapnumber].io.Out.Write(temp);
                                        break;
                                    }
                                case "reflexive":
                                    {
                                        br.BaseStream.Position = int.Parse(xtr.GetAttribute("ExtractedOffset"));
                                        byte[] temp = br.ReadBytes(int.Parse(xtr.GetAttribute("ChunkCount")) * int.Parse(xtr.GetAttribute("ChunkSize")));

                                        Maps.map[mapnumber].io.Out.BaseStream.Position = int.Parse(xtr.GetAttribute("MapOffset"));
                                        Maps.map[mapnumber].io.Out.Write(temp);
                                        break;
                                    }
                            }
                            break;
                        }
                }
            }            
        }

        public void Extract_Meta_wIFP(ref BinaryWriter bw, ref XmlTextWriter xtw, object[] ifp, int offset)
        {
            foreach (object o in ifp)
            {
                IFPIO.BaseObject tempbase = (IFPIO.BaseObject)o;
                
                switch (tempbase.ObjectType)
                {
                    case IFPIO.ObjectEnum.Reflexive:
                        IFPIO.Reflexive tempreflex = (IFPIO.Reflexive)tempbase;

                        Maps.map[mapnumber].io.In.BaseStream.Position = offset + tempreflex.offset;
                        int chunkcount = Maps.map[mapnumber].io.In.ReadInt32();
                        if (chunkcount > 0)
                        {
                            int size = chunkcount * tempreflex.chunkSize;
                            int translation = Maps.map[mapnumber].io.In.ReadInt32() - Magic;
                            Maps.map[mapnumber].io.In.BaseStream.Position = translation;
                            byte[] temp = new byte[size];
                            temp = Maps.map[mapnumber].io.In.ReadBytes(size);
                            bw.Seek((int)bw.BaseStream.Length, SeekOrigin.Begin);
                            bw.Write(temp);
                            xtw.WriteStartElement("Reflexive");
                            xtw.WriteAttributeString("Description", tempreflex.name);
                            xtw.WriteAttributeString("Offset", tempreflex.offset.ToString());
                            xtw.WriteAttributeString("ChunkCount", chunkcount.ToString());
                            xtw.WriteAttributeString("ChunkSize", tempreflex.chunkSize.ToString());
                            xtw.WriteAttributeString("MapOffset", translation.ToString());
                            int eo = (int)bw.BaseStream.Position - size;
                            xtw.WriteAttributeString("ExtractedOffset", eo.ToString());
                            xtw.WriteEndElement();
                            Extract_Meta_wIFP(ref bw, ref xtw, tempreflex.items, translation);
                        }
                        break;
                }
            }
        }
        #endregion

        public void Create_IFP(string outputfilename)
        {
            Size = 0x750;

            XmlTextWriter xtw = new XmlTextWriter(outputfilename + ".forge", Encoding.Default);
            xtw.Formatting = Formatting.Indented;

            xtw.WriteStartElement("plugin");
            xtw.WriteAttributeString("class", Class);
            xtw.WriteAttributeString("author", "shade45");
            xtw.WriteAttributeString("version", "0.0");
            xtw.WriteAttributeString("headersize", "0");

            for (int i = 0; i < Size / 4; i++)
            {
                Maps.map[mapnumber].io.In.BaseStream.Position = Offset + (i * 4);
                int temp = Maps.map[mapnumber].io.In.ReadInt32();
                if (temp > Maps.map[mapnumber].map_header.address_mask && temp < Maps.map[mapnumber].map_header.address_mask + Maps.map[mapnumber].map_header.map_size)
                {
                    xtw.WriteStartElement("struct");
                    xtw.WriteAttributeString("name", "Unknown");
                    int tempoff = ((int)Maps.map[mapnumber].io.In.BaseStream.Position - 8) - Offset;
                    xtw.WriteAttributeString("offset", tempoff.ToString());
                    xtw.WriteAttributeString("visible", "false");
                    xtw.WriteAttributeString("size", "0");
                    xtw.WriteAttributeString("label", "");
                    xtw.WriteEndElement();
                }
            }

            xtw.WriteEndElement();
            xtw.Close();
        }
        #endregion
    }
}
