using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;

using Halo3Map.IO;
using System.Windows.Forms;
using H3Dev.Map;

namespace Halo3Map.plugins
{
    public abstract class Chunk
    {

        public virtual void Read(EndianReader br)
        {
            FieldInfo[] fields = this.GetType().GetFields(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
            foreach (FieldInfo fi in fields)
            {
                if (fi.FieldType == typeof(byte))
                    fi.SetValue(this, br.ReadByte());
                else if (fi.FieldType == typeof(short))
                    fi.SetValue(this, br.ReadInt16());
                else if (fi.FieldType == typeof(ushort))
                    fi.SetValue(this, br.ReadUInt16());
                else if (fi.FieldType == typeof(int))
                    fi.SetValue(this, br.ReadInt32());
                else if (fi.FieldType == typeof(uint))
                    fi.SetValue(this, br.ReadUInt32());
                else if (fi.FieldType == typeof(long))
                    fi.SetValue(this, br.ReadInt64());
                else if (fi.FieldType == typeof(ulong))
                    fi.SetValue(this, br.ReadUInt64());
                else if (fi.FieldType == typeof(double))
                    fi.SetValue(this, br.ReadDouble());
                else if (fi.FieldType == typeof(float))
                    fi.SetValue(this, br.ReadSingle());
                else if (fi.FieldType.IsEnum)
                {
                    Type t = Enum.GetUnderlyingType(fi.FieldType);
                    if (t == typeof(byte))
                        fi.SetValue(this, br.ReadByte());
                    else if (t == typeof(short))
                        fi.SetValue(this, br.ReadInt16());
                    else if (t == typeof(int))
                        fi.SetValue(this, br.ReadInt32());
                    else if (t == typeof(long))
                        fi.SetValue(this, br.ReadInt64());
                    else
                    {
                        MessageBox.Show("Unable to read this enum type! Type: " + t.ToString());
                    }
                }
                else if (fi.FieldType == typeof(byte[]))
                {
                    int size = 0;
                    object[] attributes = fi.GetCustomAttributes(typeof(SizeAttribute), true);
                    if (attributes.Length > 0)
                        size = ((SizeAttribute)attributes[0]).Size;

                    fi.SetValue(this, br.ReadBytes(size));
                }
                else if (fi.FieldType == typeof(string))
                {
                    int size = 32;
                    object[] attributes = fi.GetCustomAttributes(typeof(SizeAttribute), true);
                    if (attributes.Length > 0)
                        size = ((SizeAttribute)attributes[0]).Size;

                    string temp = new string(br.ReadChars(size)).Replace("\0", "");
                    fi.SetValue(this, temp);
                }
                else if (fi.FieldType.IsGenericType && fi.FieldType.GetGenericTypeDefinition() == typeof(H3Reflexive<>))
                {
                    object reflexive = Activator.CreateInstance(fi.FieldType, new object[] { br });
                    fi.SetValue(this, reflexive);
                }
                else
                {
                    MessageBox.Show("Unable to read this datatype! Type: " + fi.FieldType.ToString());
                }
            }
        }

        public virtual void Write()
        {


        }

    }
}