using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Halo3Map.Data;
using System.Reflection;
using MetaEditor.Controls;
using System.Collections;

namespace MetaEditor
{
    /// <summary>
    /// A Control for viewing values of a Halo 3 Tag
    /// </summary>
    public partial class MetaView : UserControl
    {
        public H3Tag tag;
        public FieldInfo[] fields;

        public MetaView()
        {
            //this.DoubleBuffered = true;
        }

        public MetaView(H3Tag Tag)
        {
            //this.DoubleBuffered = true;

            tag = Tag;
            
            InitializeComponent();
            lblPluginType.Text += " " + tag.TagType;
            lblVersion.Text += " " + tag.Version;
            lblAuthor.Text += " " + tag.Author;                
        }
        /// <summary>
        /// Loads controls into meta viewer
        /// </summary>
        public void LoadControls()
        {
            //Load field info form H3Tag, we only want fields which are public or instance
            fields = tag.GetType().GetFields(BindingFlags.Public | BindingFlags.Instance);

            metaEditorPanel1.ClearControls();

            //For each field we just loaded
            foreach (FieldInfo fi in fields)
            {
                //Check the value type of the field
                if (fi.FieldType == typeof(byte))
                {
                    //Create an appropriate control for this value type and set its value and name
                    MetaEditor.Controls.Byte b = new MetaEditor.Controls.Byte((byte)fi.GetValue(tag), fi.Name);
                    //Make sure the control will always dock to the top of our control
                    b.Dock = DockStyle.Top;
                    //Add control to metaEditorPanel1's control list
                    metaEditorPanel1.PanelControls.Add(b);
                    //Need this to make the controls appear in appropriate order
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                if (fi.FieldType == typeof(char))
                {
                    MetaEditor.Controls.String s = new MetaEditor.Controls.String((char)fi.GetValue(tag), fi.Name);
                    s.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(s);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(short))
                {
                    Short s = new Short((short)fi.GetValue(tag), fi.Name);
                    s.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(s);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(ushort))
                {
                    Short s = new Short((ushort)fi.GetValue(tag), fi.Name);
                    s.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(s);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(int))
                {
                    Int i = new Int((int)fi.GetValue(tag), fi.Name);
                    i.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(i);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(uint))
                {
                    Int i = new Int((uint)fi.GetValue(tag), fi.Name);
                    i.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(i);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(long)) 
                {
                    Int i = new Int((long)fi.GetValue(tag), fi.Name);
                    i.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(i);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(ulong)) 
                {
                    Int i = new Int((ulong)fi.GetValue(tag), fi.Name);
                    i.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(i);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(double)) 
                {
                    Float f = new Float((double)fi.GetValue(tag), fi.Name);
                    f.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(f);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(float))
                {
                    Float f = new Float((float)fi.GetValue(tag), fi.Name);
                    f.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(f);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType.IsEnum)
                {
                    //Find out what type of enum we are dealing with
                    Type t = System.Enum.GetUnderlyingType(fi.FieldType);
                    //Act appropriatly for each type
                    if (t == typeof(byte))
                    {
                        MetaEditor.Controls.Enum e = new MetaEditor.Controls.Enum((byte)fi.GetValue(tag), System.Enum.GetNames(t), fi.Name);
                        e.Dock = DockStyle.Top;
                        metaEditorPanel1.PanelControls.Add(e);
                        metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                    }
                    else if (t == typeof(short))
                    {
                        MetaEditor.Controls.Enum e = new MetaEditor.Controls.Enum((short)fi.GetValue(tag), System.Enum.GetNames(t), fi.Name);
                        e.Dock = DockStyle.Top;
                        metaEditorPanel1.PanelControls.Add(e);
                        metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                    }
                    else if (t == typeof(int))
                    {
                        MetaEditor.Controls.Enum e = new MetaEditor.Controls.Enum((int)fi.GetValue(tag), System.Enum.GetNames(t), fi.Name);
                        e.Dock = DockStyle.Top;
                        metaEditorPanel1.PanelControls.Add(e);
                        metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                    }
                    else if (t == typeof(long))
                    {
                        MetaEditor.Controls.Enum e = new MetaEditor.Controls.Enum((long)fi.GetValue(tag), System.Enum.GetNames(t), fi.Name);
                        e.Dock = DockStyle.Top;
                        metaEditorPanel1.PanelControls.Add(e);
                        metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                    }
                }
                if (fi.FieldType == typeof(char[]))
                {
                    MetaEditor.Controls.String s = new MetaEditor.Controls.String((char[])fi.GetValue(tag), fi.Name);
                    s.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(s);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(byte[]))
                {
                    /*if (fi.Name.StartsWith("unknown", StringComparison.OrdinalIgnoreCase))
                    {
                        Unknown u = new Unknown((byte[])fi.GetValue(tag), fi.Name);
                        u.Dock = DockStyle.Top;
                        metaEditorPanel1.PanelControls.Add(u);
                        metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                    }*/
                    //MetaEditor.Controls.Byte b = new MetaEditor.Controls.Byte((byte[])fi.GetValue(tag), fi.Name);
                    //b.Dock = DockStyle.Top;
                    //metaEditorPanel1.PanelControls.Add(b);
                    //metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(string))
                {
                    MetaEditor.Controls.String s = new MetaEditor.Controls.String((string)fi.GetValue(tag), fi.Name);
                    s.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(s);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(Halo3Map.Data.StringID))
                {
                    Halo3Map.Data.StringID si = (Halo3Map.Data.StringID)fi.GetValue(tag);
                    MetaEditor.Controls.StringID s = new MetaEditor.Controls.StringID(si.Index, fi.Name, tag.Map);
                    s.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(s);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType == typeof(TagReference))
                {
                    TagReference t = (TagReference)fi.GetValue(tag);
                    Ident i = new Ident(t.Ident, fi.Name, tag.Map);
                    i.Dock = DockStyle.Top;
                    metaEditorPanel1.PanelControls.Add(i);
                    metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                }
                else if (fi.FieldType.IsGenericType && fi.FieldType.GetGenericTypeDefinition() == typeof(H3Reflexive<>))
                {
                    IList list = (IList)fi.GetValue(tag);                    
                    if (list.Count != 0)
                    {
                        Reflexive r = new Reflexive(list, fi.Name, tag.Map);
                        r.Dock = DockStyle.Top;
                        metaEditorPanel1.PanelControls.Add(r);
                        metaEditorPanel1.PanelControls[metaEditorPanel1.PanelControls.Count - 1].BringToFront();
                    }
                }
            }
        }
        /// <summary>
        /// Writes the values of H3Tag back to the map file
        /// </summary>
        private void btnSave_Click_1(object sender, EventArgs e)
        {
            Save();            
            tag.Write(tag.Map.IO.Out);
            MessageBox.Show("Success", "Done");
        }  
        /// <summary>
        /// Writes values of meteviewer controls back to our H3Tag
        /// </summary>
        public void Save()
        {
            //For every field in our H3Tag
            foreach (FieldInfo fi in fields)
            {
                //And every control in the meta editor
                foreach (Control c in metaEditorPanel1.Controls)
                {
                    //Check which control holds the value for which field
                    if (c.GetType() == typeof(MetaEditor.Controls.Byte) && ((MetaEditor.Controls.Byte)c).ReturnName() == fi.Name)
                    {
                        //If a match is found set new value
                        fi.SetValue(tag, ((MetaEditor.Controls.Byte)c).ReturnValue());
                    }
                    else if (c.GetType() == typeof(Short) && ((Short)c).ReturnName() == fi.Name)
                    {
                        fi.SetValue(tag, ((Short)c).ReturnValue());
                    }
                    else if (c.GetType() == typeof(Int) && ((Int)c).ReturnName() == fi.Name)
                    {
                        fi.SetValue(tag, ((Int)c).ReturnValue());
                    }
                    else if (c.GetType() == typeof(Float) && ((Float)c).ReturnName() == fi.Name)
                    {
                        fi.SetValue(tag, ((Float)c).ReturnValue());
                    }
                    else if(c.GetType() == typeof(MetaEditor.Controls.Enum) && ((MetaEditor.Controls.Enum)c).ReturnName() == fi.Name)
                    {
                        //Checks what type of enum we are dealing with
                        if (((MetaEditor.Controls.Enum)c).type == typeof(byte))
                        {
                            //If a valid value type is found new value of enum
                            fi.SetValue(tag, ((MetaEditor.Controls.Enum)c).ReturnByteValue());
                        }
                        if (((MetaEditor.Controls.Enum)c).type == typeof(short))
                        {
                            fi.SetValue(tag, ((MetaEditor.Controls.Enum)c).ReturnShortValue());
                        }
                        if (((MetaEditor.Controls.Enum)c).type == typeof(int))
                        {
                            fi.SetValue(tag, ((MetaEditor.Controls.Enum)c).ReturnIntValue());
                        }
                        if (((MetaEditor.Controls.Enum)c).type == typeof(long))
                        {
                            fi.SetValue(tag, ((MetaEditor.Controls.Enum)c).ReturnLongValue());
                        }
                    }
                    else if (c.GetType() == typeof(MetaEditor.Controls.String) && ((MetaEditor.Controls.String)c).ReturnName() == fi.Name)
                    {
                        if (((MetaEditor.Controls.String)c).t == typeof(string))
                            fi.SetValue(tag, ((MetaEditor.Controls.String)c).ReturnStringValue());
                        else if (((MetaEditor.Controls.String)c).t == typeof(char))
                            fi.SetValue(tag, ((MetaEditor.Controls.String)c).ReturnCharValue());
                        else if(((MetaEditor.Controls.String)c).t == typeof(char))
                            fi.SetValue(tag, ((MetaEditor.Controls.String)c).ReturnCharArrayValue());

                    }
                    else if (c.GetType() == typeof(Ident) && ((Ident)c).ReturnName() == fi.Name)
                    {
                        fi.SetValue(tag, ((Ident)c).ReturnValue());
                    }
                    else if (c.GetType() == typeof(Reflexive) && ((Reflexive)c).ReturnName() == fi.Name)
                    {
                        fi.SetValue(tag, ((Reflexive)c).ReturnValue());
                    }
                }
            }
        }
        /// <summary>
        /// Takes a snapshot of the meta editor and saves it a a JPEG imagle file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void screenShotMetaEditorToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Save file dialog to choose where to save file
            SaveFileDialog sfd = new SaveFileDialog();
            //What extension tipe will the file be
            sfd.Filter = "JPEG Image (*.jpg)|*.jpg";
            //If user doesnt choose "cancel" and there are no other errors continue
            if(sfd.ShowDialog() == DialogResult.OK)
            {
                //Create a new bitmap to hold our image data
                Bitmap bi = new Bitmap(metaEditorPanel1.Width, metaEditorPanel1.Height);
                //Draw contents of panel1 to the bitmap just created
                metaEditorPanel1.DrawToBitmap(bi, new Rectangle(0, 0, bi.Width, bi.Height));
                //Save bitmap to file as a JPEG
                bi.Save(sfd.FileName, System.Drawing.Imaging.ImageFormat.Jpeg);
            }
        }   
    }
}